var _ = require('lodash');
var BaseModel = require("./../../../baseModel");
var Encryption = require("./../../../utils").Encryption;

function thisModule() {

    var generateDeviceId = function () {
        return Encryption.md5(Encryption.randHash());
    };

    var generateSessionId = function () {
        return Encryption.randHash();
    };

    var setSessionId = function () {
        return {'session.id': generateSessionId(), 'session.created': Date.now()};
    };

    var cultureCodeSetter = function (value) {
        var culture = {};
        culture.language = value.substring(0, 2).toLowerCase();
        culture.region = value.substring(3, 5).toLowerCase();
        return {'culture.language': culture.language, 'culture.region': culture.region};
    };

    var setHeightWidth = function (value) {
        var dimensions = value.split("x");
        return {'device.screen.height': dimensions[0], 'device.screen.width': dimensions[1]};
    };

    var setTestMessage = function (value) {
        return {'message': 'This is a test message'};
    };

    var setDeviceType = function (value) {

        var type = 'desktop';

        if (/(Android)/i.test(value)) {
            if (/(Mobile)/i.test(value)) {
                type = 'phone';
            }
            else {
                type = 'tablet';
            }

            if (/(Mobile)/i.test(value) && /(Kindle)/i.test(value)) {
                type = 'tablet';
            }
        }
        else {
            if (/(iP(ad)|Kindle|Playbook|Silk-Accelerated)/i.test(value)) {
                type = 'tablet';
            }
            if (/(Mobile|iP(hone|od)|Windows Phone|BlackBerry)/i.test(value)) {
                type = 'phone';
            }
        }
        return {'device.type': type}
    };

    var setOsTypeAndVersion = function (value) {
        var os
            , version;

        // Get Android OS Version
        var regExAndroid = /(Android)\s?([\d|\.]*)/i;
        if (regExAndroid.test(value)) {
            var partsArray = regExAndroid.exec(value);
            os = 'android';
            version = partsArray[2];
        }

        //  Get iOS OS Version
        var regExiOs = /(iP(hone|ad|od))\s?OS\s?([\d|_]*)/i;
        if (regExiOs.test(value)) {
            var partsArray = regExiOs.exec(value);
            os = 'ios';
            version = partsArray[3].replace(/_/g, '.');
        }

        //  Get Windows Phone OS Version
        var regExWP = /(Windows Phone)\s?OS\s?([\d|.]*)/i;
        if (regExWP.test(value)) {
            var partsArray = regExWP.exec(value);
            os = 'windows phone';
            version = partsArray[2];
        }
        return {'device.os.type': os, 'device.os.version': version};
    };

    var setDevice = function (value) {
        var type = setDeviceType(value);
        var os = setOsTypeAndVersion(value);
        return _.assign(type, os);
    };

    var trimSpaces = function (value) {
        return {this: value.replace(' ', '')};
    };

    var model = BaseModel.extend({
            data: {

                /** id: Device-Id generated by device */
                id: {
                    type: String,
                    maxLength: 32,
                    minLength: 32,
                    required: true,
                    unique: true,
                    index: true,
                    convert: 'lower',
                    default: generateDeviceId
                },

                session: {
                    set: {
                        type: Boolean,
                        virtual: setSessionId,
                        public: {set: true, get: false}
                    },
                    id: {
                        type: String,
                        maxLength: 64,
                        minLength: 64,
                        unique: true,
                        sparse: true,
                        index: true,
                        convert: 'lower',
                        public: {set: false, get: true}
                    },
                    cidr: {
                        type: 'CIDR',
                        index: true
                    },
                    ip: {
                        type: 'IP',
                        index: true
                    },
                    expireable: {
                        type: Boolean,
                        default: true,
                        index: true
                    },
                    created: {
                        type: Date,
                        public: {set: false, get: true},
                        index: true
                    }
                },
                culture: {
                    language: {
                        type: String,
                        minLength: 2,
                        maxLength: 2,
                        match: /[a-zA-Z]{2}/i,
                        required: true,
                        public: {set: false, get: true}
                    },
                    region: {
                        type: String,
                        minLength: 2,
                        maxLength: 2,
                        match: /[a-zA-Z]{2}/i,
                        required: true,
                        public: {set: false, get: true}
                    },
                    code: {
                        type: String,
                        minLength: 5,
                        maxLength: 5,
                        match: /[a-zA-Z]{2}-[a-zA-Z]{2}/i,
                        required: true,
                        public: {set: true, get: false},
                        virtual: cultureCodeSetter,
                        default: 'de-de'
                    }
                },

                app: {
                    id: {
                        type: String,
                        minLength: 5,
                        maxLength: 50,
                        allow: ['12Gebrauchtwagen', 'Mr.Crown'],
                        virtual: trimSpaces
                    },
                    version: {
                        type: String,
                        minLength: 1,
                        maxLength: 5,
                        match: /^([\d]{1,3}\.)?([\d]{1,3}\.)?([\d]{1,3}\.)?[\d]{1,3}$/i //Match up to 1.2.3.4
                    },
                    vendor: {
                        id: {
                            type: String,
                            minLength: 5,
                            maxLength: 50,
                            convert: 'lower'
                        }
                    },

                    advertiser: {
                        id: {
                            type: String,
                            minLength: 5,
                            maxLength: 50,
                            convert: 'lower'
                        }
                    }
                },

                device: {
                    type: {
                        type: String,
                        allow: ['phone', 'tablet', 'desktop'],
                        public: {set: false, get: true}
                    },
                    userAgent: {
                        type: String,
                        minLength: 5,
                        convert: 'lower',
                        virtual: setDevice
                    },
                    os: {
                        type: {
                            type: String,
                            minLength: 2,
                            maxLength: 50,
                            allow: ['ios', 'android', 'windows phone'],
                            convert: 'lower'
                        },
                        version: {
                            type: String,
                            minLength: 1,
                            maxLength: 5,
                            match: /(\d|.)*/i //Match up to 1.2.3.4
                        }
                    },
                    carrier: {
                        type: {
                            type: String,
                            minLength: 5,
                            maxLength: 50,
                            convert: 'lower'
                        }
                    },
                    screen: {
                        height: {
                            type: Number,
                            minLength: 1,
                            maxLength: 6,
                            public: {set: false, get: true}
                        },
                        width: {
                            type: Number,
                            minLength: 1,
                            maxLength: 6,
                            public: {set: false, get: true}
                        },
                        resolution: {
                            type: String,
                            match: /(\d+)x(\d+)/i,
                            virtual: setHeightWidth,
                            public: {set: true, get: false}
                        }
                    }
                },

                created: {
                    type: Date,
                    default: Date.now(),
                    required: true
                },
                lastModified: {
                    type: Date,
                    default: Date.now(),
                    required: true
                },
                status: {
                    type: String,
                    default: 'inactive',
                    allow:['active','inactive','disabled'],
                    required: true
                },

                //Test submodel
                messages: [
                    {
                        name: {
                            type: String,
                            convert: 'upper',
                            virtual: setTestMessage,
                            index: true
                        },
                        message: {
                            default: 'invalid',
                            required: true
                        },
                        send: {
                            type: Boolean,
                            public: {set: false, get: true}
                        }

                    }
                ],
                customData: [],
                nameOfChildren: {
                    type: Array,
                    subType: String,
                    convert: 'upper'
                },
                mustbethere: {
                    required: true
                },
                maxvalues: {
                    max: 20
                },
                minvalues: {
                    min: 2
                },
                intValue: {
                    type: 'integer'
                },
                safeIntValue: {
                    type: 'safeinteger'
                },
                allowMultipleValues1: {
                    type: String,
                    allowMultiple: [1, 2, 3, 4, 5, 6]
                },
                allowMultipleValues2: {
                    type: Number,
                    allowMultiple: [1, 2, 3, 4, 5, 6]
                },
                allowMultipleValues3: {
                    type: String,
                    allowMultiple: [1, 2, 'testAllowCAPS']
                },
                allowMultipleValues4: {
                    type: String,
                    allowMultiple: [1, 2, 'testvaluecaps']
                }
            }
        },
        {
            disabled: false, // Enable /disable model
            identity: 'device', // Model name
            version: '1.0', // Version number
            created: '2014-01-09T19:00:00', // Creation date
            modified: '2014-01-09T19:00:00', // Last modified date
            collectionName: 'devices'
        });

    return model;
};

module.exports = thisModule();